@@ -37,11 +37,17 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QInputDialog>
 #include <QVBoxLayout>
 #include <QColorDialog>
+#include <QPrinter>
 #include <QGraphicsRectItem>
 #include "rtabmap/utilite/ULogger.h"
 #include "rtabmap/gui/KeypointItem.h"
 #include "rtabmap/core/util2d.h"
 
+#include <QtGlobal>
+#if QT_VERSION >= 0x050000
+	#include <QStandardPaths>
+#endif
+
 namespace rtabmap {
 
 //LineItem
@@ -62,7 +68,11 @@ public:
 		delete _placeHolder;
 	}
 
-	void setColor(const QColor & color);
+	void setWidth(int width)
+	{
+		_width = width;
+		this->setPen(QPen(pen().color(), _width));
+	}
 
 protected:
 	virtual void hoverEnterEvent ( QGraphicsSceneHoverEvent * event )
@@ -158,16 +168,30 @@ QIcon ImageView::createIcon(const QColor & color)
 
 ImageView::ImageView(QWidget * parent) :
 		QWidget(parent),
-		_savedFileName((QDir::homePath()+ "/") + "picture" + ".png"),
 		_alpha(100),
 		_featuresSize(0.0f),
+		_linesWidth(0),
 		_defaultBgColor(Qt::black),
 		_defaultFeatureColor(Qt::yellow),
 		_defaultMatchingFeatureColor(Qt::magenta),
 		_defaultMatchingLineColor(Qt::cyan),
+		_depthColorMapRange(0),
 		_imageItem(0),
 		_imageDepthItem(0)
 {
+#if QT_VERSION >= 0x050000
+	_savedFileName = QStandardPaths::writableLocation(QStandardPaths::PicturesLocation);
+#endif
+	QDir dir;
+	if(!dir.exists(_savedFileName))
+	{
+		_savedFileName = QDir::homePath()+ "/picture.png";
+	}
+	else
+	{
+		_savedFileName += "/picture.png";
+	}
+
 	_graphicsView = new QGraphicsView(this);
 	_graphicsView->setTransformationAnchor(QGraphicsView::AnchorUnderMouse);
 	_graphicsView->setScene(new QGraphicsScene(this));
@@ -192,6 +216,7 @@ ImageView::ImageView(QWidget * parent) :
 	_showLines = _featureMenu->addAction(tr("Show lines"));
 	_showLines->setCheckable(true);
 	_showLines->setChecked(true);
+	_setLinesWidth = _featureMenu->addAction(tr("Set lines width..."));
 	_setFeatureColor = _featureMenu->addAction(tr("Set default feature color"));
 	_setFeatureColor->setIcon(createIcon(_defaultFeatureColor));
 	_setFeatureColor->setIconVisibleInMenu(true);
@@ -233,11 +258,13 @@ ImageView::ImageView(QWidget * parent) :
 	_colorMapBlueToRed = colorMap->addAction(tr("Blue to red"));
 	_colorMapBlueToRed->setCheckable(true);
 	_colorMapBlueToRed->setChecked(false);
+	_colorMapRange = colorMap->addAction(tr("Max Range..."));
 	group = new QActionGroup(this);
 	group->addAction(_colorMapWhiteToBlack);
 	group->addAction(_colorMapBlackToWhite);
 	group->addAction(_colorMapRedToBlue);
 	group->addAction(_colorMapBlueToRed);
+	group->addAction(_colorMapRange);
 	_saveImage = _menu->addAction(tr("Save picture..."));
 	_saveImage->setEnabled(false);
 
@@ -259,6 +286,7 @@ void ImageView::saveSettings(QSettings & settings, const QString & group) const
 	settings.setValue("features_shown", this->isFeaturesShown());
 	settings.setValue("features_size", this->getFeaturesSize());
 	settings.setValue("lines_shown", this->isLinesShown());
+	settings.setValue("lines_width", this->getLinesWidth());
 	settings.setValue("alpha", this->getAlpha());
 	settings.setValue("bg_color", this->getDefaultBackgroundColor());
 	settings.setValue("feature_color", this->getDefaultFeatureColor());
@@ -268,6 +296,7 @@ void ImageView::saveSettings(QSettings & settings, const QString & group) const
 	settings.setValue("graphics_view_scale", this->isGraphicsViewScaled());
 	settings.setValue("graphics_view_scale_to_height", this->isGraphicsViewScaledToHeight());
 	settings.setValue("colormap", _colorMapWhiteToBlack->isChecked()?0:_colorMapBlackToWhite->isChecked()?1:_colorMapRedToBlue->isChecked()?2:3);
+	settings.setValue("colormap_range", this->getDepthColorMapRange());
 	if(!group.isEmpty())
 	{
 		settings.endGroup();
@@ -285,6 +314,7 @@ void ImageView::loadSettings(QSettings & settings, const QString & group)
 	this->setFeaturesShown(settings.value("features_shown", this->isFeaturesShown()).toBool());
 	this->setFeaturesSize(settings.value("features_size", this->getFeaturesSize()).toInt());
 	this->setLinesShown(settings.value("lines_shown", this->isLinesShown()).toBool());
+	this->setLinesWidth(settings.value("lines_width", this->getLinesWidth()).toInt());
 	this->setAlpha(settings.value("alpha", this->getAlpha()).toInt());
 	this->setDefaultBackgroundColor(settings.value("bg_color", this->getDefaultBackgroundColor()).value<QColor>());
 	this->setDefaultFeatureColor(settings.value("feature_color", this->getDefaultFeatureColor()).value<QColor>());
@@ -298,6 +328,7 @@ void ImageView::loadSettings(QSettings & settings, const QString & group)
 	_colorMapBlackToWhite->setChecked(colorMap==1);
 	_colorMapRedToBlue->setChecked(colorMap==2);
 	_colorMapBlueToRed->setChecked(colorMap==3);
+	this->setDepthColorMapRange(settings.value("colormap_range", this->getDepthColorMapRange()).toFloat());
 	if(!group.isEmpty())
 	{
 		settings.endGroup();
@@ -361,6 +392,11 @@ const QColor & ImageView::getBackgroundColor() const
 	return _graphicsView->backgroundBrush().color();
 }
 
+float ImageView::getDepthColorMapRange() const
+{
+	return _depthColorMapRange;
+}
+
 uCvQtDepthColorMap ImageView::getDepthColorMap() const
 {
 	uCvQtDepthColorMap colorMap = uCvQtDepthWhiteToBlack;
@@ -658,6 +694,12 @@ void ImageView::setBackgroundColor(const QColor & color)
 	}
 }
 
+void ImageView::setDepthColorMapRange(float value)
+{
+	_depthColorMapRange = value;
+}
+
+
 void ImageView::computeScaleOffsets(const QRect & targetRect, float & scale, float & offsetX, float & offsetY) const
 {
 	scale = 1.0f;
@@ -762,9 +804,8 @@ void ImageView::paintEvent(QPaintEvent *event)
 			{
 				for(QList<QGraphicsLineItem*>::iterator iter = _lines.begin(); iter != _lines.end(); ++iter)
 				{
-					QColor color = (*iter)->pen().color();
 					painter.save();
-					painter.setPen(color);
+					painter.setPen(QPen((*iter)->pen().color(), _linesWidth));
 					painter.drawLine((*iter)->line());
 					painter.restore();
 				}
@@ -806,11 +847,16 @@ void ImageView::contextMenuEvent(QContextMenuEvent * e)
 		if(!_graphicsView->scene()->sceneRect().isNull())
 		{
 			QString text;
+			QString extensions = "*.png *.xpm *.jpg";
+			if(_graphicsView->isVisible())
+			{
+				extensions += " *.pdf";
+			}
 #ifdef QT_SVG_LIB
-			text = QFileDialog::getSaveFileName(this, tr("Save figure to ..."), _savedFileName, "*.png *.xpm *.jpg *.pdf *.svg");
-#else
-			text = QFileDialog::getSaveFileName(this, tr("Save figure to ..."), _savedFileName, "*.png *.xpm *.jpg *.pdf");
+			extensions += " *.svg";
 #endif
+			text = QFileDialog::getSaveFileName(this, tr("Save figure to ..."), _savedFileName, extensions);
+
 			if(!text.isEmpty())
 			{
 				if(QFileInfo(text).suffix() == "")
@@ -820,17 +866,34 @@ void ImageView::contextMenuEvent(QContextMenuEvent * e)
 				}
 
 				_savedFileName = text;
-				QImage img(_graphicsView->sceneRect().width(), _graphicsView->sceneRect().height(), QImage::Format_ARGB32_Premultiplied);
-				QPainter p(&img);
-				if(_graphicsView->isVisible())
+				if(QFileInfo(text).suffix().compare("pdf") == 0)
 				{
+					QPrinter printer(QPrinter::HighResolution);
+					printer.setOrientation(QPrinter::Portrait);
+					printer.setOutputFileName( text );
+					QPainter p(&printer);
+					p.begin(&printer);
+					double xscale = printer.pageRect().width()/double(_graphicsView->sceneRect().width());
+					double yscale = printer.pageRect().height()/double(_graphicsView->sceneRect().height());
+					double scale = qMin(xscale, yscale);
+					p.scale(scale, scale);
 					_graphicsView->scene()->render(&p, _graphicsView->sceneRect(), _graphicsView->sceneRect());
+					p.end();
 				}
 				else
 				{
-					this->render(&p, QPoint(), _graphicsView->sceneRect().toRect());
+					QImage img(_graphicsView->sceneRect().width(), _graphicsView->sceneRect().height(), QImage::Format_ARGB32_Premultiplied);
+					QPainter p(&img);
+					if(_graphicsView->isVisible())
+					{
+						_graphicsView->scene()->render(&p, _graphicsView->sceneRect(), _graphicsView->sceneRect());
+					}
+					else
+					{
+						this->render(&p, QPoint(), _graphicsView->sceneRect().toRect());
+					}
+					img.save(text);
 				}
-				img.save(text);
 			}
 		}
 	}
@@ -910,6 +973,16 @@ void ImageView::contextMenuEvent(QContextMenuEvent * e)
 			this->setImageDepth(_imageDepthCv);
 		Q_EMIT configChanged();
 	}
+	else if(action == _colorMapRange)
+	{
+		bool ok = false;
+		double value = QInputDialog::getDouble(this, tr("Set depth colormap max range"), tr("Range (m), 0=no limit"), _depthColorMapRange, 0, 9999, 1, &ok);
+		if(ok)
+		{
+			this->setDepthColorMapRange(value);
+			Q_EMIT configChanged();
+		}
+	}
 	else if(action == _setAlpha)
 	{
 		bool ok = false;
@@ -930,6 +1003,16 @@ void ImageView::contextMenuEvent(QContextMenuEvent * e)
 			Q_EMIT configChanged();
 		}
 	}
+	else if(action == _setLinesWidth)
+	{
+		bool ok = false;
+		int value = QInputDialog::getInt(this, tr("Set lines width"), tr("Width"), _linesWidth, 0, 999, 1, &ok);
+		if(ok)
+		{
+			this->setLinesWidth(value);
+			Q_EMIT configChanged();
+		}
+	}
 
 	if(action == _showImage || action ==_showImageDepth)
 	{
@@ -1046,6 +1129,7 @@ void ImageView::addLine(float x1, float y1, float x2, float y2, QColor color, co
 	color.setAlpha(this->getAlpha());
 	LineItem * item  = new LineItem(x1, y1, x2, y2, text);
 	item->setPen(QPen(color));
+	item->setWidth(_linesWidth);
 	_lines.push_back(item);
 	item->setVisible(isLinesShown());
 	item->setZValue(1);
@@ -1086,7 +1170,7 @@ void ImageView::setImage(const QImage & image)
 void ImageView::setImageDepth(const cv::Mat & imageDepth)
 {
 	_imageDepthCv = imageDepth;
-	setImageDepth(uCvMat2QImage(_imageDepthCv, true, getDepthColorMap()));
+	setImageDepth(uCvMat2QImage(_imageDepthCv, true, getDepthColorMap(), 0.0f, _depthColorMapRange));
 }
 
 void ImageView::setImageDepth(const QImage & imageDepth)
@@ -1207,6 +1291,22 @@ void ImageView::setFeaturesSize(int size)
 	}
 }
 
+void ImageView::setLinesWidth(int width)
+{
+	_linesWidth = width;
+	for(QList<QGraphicsLineItem*>::iterator iter=_lines.begin(); iter!=_lines.end(); ++iter)
+	{
+		if(dynamic_cast<LineItem*>(*iter))
+		{
+			((LineItem*)(*iter))->setWidth(_linesWidth);
+		}
+	}
+	if(!_graphicsView->isVisible())
+	{
+		this->update();
+	}
+}
+
 void ImageView::setSceneRect(const QRectF & rect)
 {
 	_graphicsView->scene()->setSceneRect(rect);
